package cn.iocoder.yudao.module.system.service.robot.camera;

import java.util.*;
import javax.validation.*;
import javax.validation.constraints.NotEmpty;

import cn.iocoder.yudao.module.system.controller.admin.robot.camera.RobotCameraAddVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.camera.RobotCameraPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.camera.RobotCameraSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotCameraDO;
import com.baomidou.mybatisplus.extension.service.IService;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;

/**
 * 车辆摄像头信息 Service 接口
 *
 * @author 陈宾顺
 */
public interface RobotCameraService extends IService<RobotCameraDO> {

    /**
     * 创建车辆摄像头信息
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCamera(@Valid RobotCameraSaveReqVO createReqVO);

    /**
     * 更新车辆摄像头信息
     *
     * @param updateReqVO 更新信息
     */
    void updateCamera(@Valid RobotCameraSaveReqVO updateReqVO);

    /**
     * 删除车辆摄像头信息
     *
     * @param id 编号
     */
    void deleteCamera(Long id);

    /**
     * 获得车辆摄像头信息
     *
     * @param id 编号
     * @return 车辆摄像头信息
     */
    RobotCameraDO getCamera(Long id);

    /**
     * 获得车辆摄像头信息分页
     *
     * @param pageReqVO 分页查询
     * @return 车辆摄像头信息分页
     */
    PageResult<RobotCameraDO> getCameraPage(RobotCameraPageReqVO pageReqVO);

    /**
     * 校验摄像头IP和端口是否存在
     * @param cameraAddVOList
     */
    void checkCameraExist(List<RobotCameraAddVO> cameraAddVOList);

    /**
     * 添加摄像头
     * @param cameraAddVOList
     * @param robotNo
     */
    void createCameraList(List<RobotCameraAddVO> cameraAddVOList, String robotNo);

    /**
     * 根据车辆编号删除摄像头
     * @param robotNo
     */
    void deleteCameraByRobotNo(String robotNo);

    /**
     * 根据机器人编号查询
     * @param robotNo
     * @return
     */
    List<RobotCameraAddVO> getCameraByRobotNo(String robotNo);
}
