package cn.iocoder.yudao.module.system.service.robot.pathplanning;

import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.util.date.DateUtils;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.framework.tenant.core.context.TenantContextHolder;
import cn.iocoder.yudao.module.mqtt.api.common.CommonApi;
import cn.iocoder.yudao.module.mqtt.api.path.task.TaskLimitationAreaDTO;
import cn.iocoder.yudao.module.mqtt.api.path.task.TaskRobotNoLimittationAreaDTO;
import cn.iocoder.yudao.module.mqtt.api.path.task.TaskToPathPlanningDTO;
import cn.iocoder.yudao.module.system.constant.path.PathPlanningChcheConstant;
import cn.iocoder.yudao.module.system.constant.path.PathPlanningTopicConstant;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotTaskDetailAddVO;
import cn.iocoder.yudao.module.system.dal.dataobject.houselocation.WareHouseLocationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapDO;
import cn.iocoder.yudao.module.system.dal.dataobject.positionmap.PositionMapItemDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotChargeLogDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotTaskDetailDO;
import cn.iocoder.yudao.module.system.dal.mysql.houselocation.WareHouseLocationMapper;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapItemMapper;
import cn.iocoder.yudao.module.system.dal.mysql.positionmap.PositionMapMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskDetailMapper;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotTaskMapper;
import cn.iocoder.yudao.module.system.enums.common.ZeroOneEnum;
import cn.iocoder.yudao.module.system.enums.item.PositionMapItemEnum;
import cn.iocoder.yudao.module.system.enums.item.UseStatusEnum;
import cn.iocoder.yudao.module.system.enums.path.PathTaskTypeEnum;
import cn.iocoder.yudao.module.system.enums.path.PathTaskTypeToRobotEnum;
import cn.iocoder.yudao.module.system.enums.redis.RobotCacheLockEnum;
import cn.iocoder.yudao.module.system.enums.robot.*;
import cn.iocoder.yudao.module.system.enums.robot.charge.ChargeModelEnum;
import cn.iocoder.yudao.module.system.enums.robot.information.ChargeTypeEnum;
import cn.iocoder.yudao.module.system.service.robot.job.DistributeTasksService;
import cn.iocoder.yudao.module.system.util.redis.RedisUtil;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Validated
@Slf4j
public class RobotPathPlanningServiceImpl implements RobotPathPlanningService {

    @Autowired
    private RobotTaskDetailMapper robotTaskDetailMapper;

    @Resource
    private PositionMapItemMapper positionMapItemMapper;

    @Value("${zn.task_need_single:true}")
    private Boolean taskNeedSingle;

    @Value("${zn.location_number_reduce:100000000}")
    private Long locationNumberReduce;

    @Resource
    private WareHouseLocationMapper locationMapper;

    @Resource
    private RedisUtil redisUtil;

    @Autowired
    private DistributeTasksService distributeTasksService;

    @Resource
    private PositionMapMapper positionMapMapper;

    @Value("${zn.robot_chearg.release_location_number_config:50}")
    private Long releaseLocationNumberConfig;

    @Value("${zn.robot_chearg.priority_config:50}")
    private Long priorityConfig;

    @Value("${zn.robot_config.offset_height}")
    private Double offsetHeight;

    @Value("${zn.path_planning.task_chche_time:604800}")
    private Long taskChcheTime;

    @Value("${zn.move-no:MOVE}")
    private String moveTaskNo;

    @Value("${zn.robot_config.default_tray_height:0.82}")
    private Double defaultTrayHeight;

    @Autowired
    private RobotTaskMapper robotTaskMapper;

    @Resource
    private CommonApi commonApi;

    /**
     * 下发任务给PP
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void sendTaskToPP() {
        TenantContextHolder.setTenantId(1L);

        log.info("-------循环任务开始查找车子和任务------");
        Pair<List<RobotInformationDO>, List<RobotTaskDetailDO>> robotAndTaskDetails =
                distributeTasksService.getRobotAndTaskDetails();

        List<RobotInformationDO> robots = robotAndTaskDetails.getLeft();
        if (ObjectUtil.isEmpty(robots)) {
            return;
        }

        List<RobotTaskDetailDO> taskDetailDOS = robotAndTaskDetails.getRight();
        if (ObjectUtil.isEmpty(taskDetailDOS)) {
            log.info("--不存在需要执行的任务--派车辆去等待点");
            moveRobotToWait(robots);
            return;
        }

        //停车不锁车
        taskDetailDOS = doPark(taskDetailDOS);
        if (ObjectUtil.isEmpty(taskDetailDOS)) {
            log.info("--不存在需要执行的任务---");
            return;
        }

        log.info("-------查找到车子和任务------");
        distributeTasksToPP(robots, taskDetailDOS);

    }

    /**
     * 处理停车不锁车的任务
     *
     * @param taskDetailDOS
     * @return
     */
    private List<RobotTaskDetailDO> doPark(List<RobotTaskDetailDO> taskDetailDOS) {
        List<RobotTaskDetailDO> parkList = new ArrayList<>();
        List<RobotTaskDetailDO> detailList = new ArrayList<>();
        for (RobotTaskDetailDO v : taskDetailDOS) {
            if (RobotTaskTypeEnum.PARK.getType().equals(v.getTaskType())
                    && ZeroOneEnum.ZERO.getType().equals(v.getNeedLock())) {
                parkList.add(v);
            } else if (!RobotTaskTypeEnum.CHARGE.getType().equals(v.getTaskType())) {
                detailList.add(v);
            }
        }

        if (ObjectUtil.isNotEmpty(parkList)) {
            List<Long> detailIds = parkList.stream().map(RobotTaskDetailDO::getId).collect(Collectors.toList());
            List<RobotTaskDetailDO> taskDetails = robotTaskDetailMapper.selectBatchIds(detailIds);
            List<Long> taskIds = taskDetails.stream().map(RobotTaskDetailDO::getRobotTaskId).collect(Collectors.toList());
            robotTaskDetailMapper.updateDoneByIds(detailIds);
            robotTaskMapper.updateDoneByIds(taskIds);
        }

        return detailList;
    }

    /**
     * 派车辆去等待点
     *
     * @param robots
     */
    private void moveRobotToWait(List<RobotInformationDO> robots) {

        robots = robots.stream()
                .filter(v -> v.getRobotStatus().equals(RobotStatusEnum.STAND_BY.getType()))
                .collect(Collectors.toList());
        if (ObjectUtil.isEmpty(robots)) {
            log.info("------没有空闲的机器人可以去等待点-----");
            return;
        }

        List<PositionMapItemDO> positionMapItems = positionMapItemMapper.selectList(new LambdaQueryWrapperX<PositionMapItemDO>()
                .eq(PositionMapItemDO::getType, PositionMapItemEnum.STOP.getType()));

        if (ObjectUtil.isEmpty(positionMapItems)) {
            log.info("------没有设置停车点，无法派发空闲的车辆去停车-----");
            return;
        }

        List<String> robotNos = robots.stream().map(RobotInformationDO::getRobotNo).collect(Collectors.toList());


        List<PositionMapItemDO> existMapItems = positionMapItems.stream()
                .filter(v -> ObjectUtil.isNotEmpty(v.getRobotNo()) && robotNos.contains(v.getRobotNo()))
                .collect(Collectors.toList());

        //车辆直接分配的停车点集合
        List<PositionMapItemDO> robotMapItems = new ArrayList<>();
        if (ObjectUtil.isNotEmpty(existMapItems)) {

            for (PositionMapItemDO positionMapItem : positionMapItems) {
                robotNos.removeIf(v -> v.equals(positionMapItem.getRobotNo()));
                if (UseStatusEnum.USEING.getType().equals(positionMapItem.getUseStatus())) {
                    continue;
                }
                positionMapItem.setUseStatus(UseStatusEnum.PRE_OCCUPANCY.getType());
                robotMapItems.add(positionMapItem);
            }
        }

        if (ObjectUtil.isNotEmpty(robotNos)) {
            List<PositionMapItemDO> emptyMapItems = positionMapItems.stream()
                    .filter(v -> ObjectUtil.isEmpty(v.getRobotNo()))
                    .collect(Collectors.toList());
            if (ObjectUtil.isNotEmpty(emptyMapItems)) {
                for (PositionMapItemDO positionMapItem : emptyMapItems) {
                    if (ObjectUtil.isEmpty(robotNos)) {
                        break;
                    }
                    positionMapItem.setRobotNo(robotNos.get(0));
                    robotNos.removeIf(v -> v.equals(positionMapItem.getRobotNo()));
                    positionMapItem.setUseStatus(UseStatusEnum.PRE_OCCUPANCY.getType());
                    robotMapItems.add(positionMapItem);
                }
            }
        }

        if (ObjectUtil.isEmpty(robotMapItems)) {
            log.info("车辆找不到合适的停车点,或者车辆都在停车点了,或者停车点不够用");
            return;
        }

        Map<String, PositionMapItemDO> positionMap =
                robotMapItems.stream().collect(Collectors.toMap(PositionMapItemDO::getRobotNo, Function.identity()));
        Map<Long, Long> sortMap = positionMapItems.stream().collect(Collectors.toMap(PositionMapItemDO::getId, PositionMapItemDO::getSortNum));


        //机器人不能行走的区域
        List<TaskRobotNoLimittationAreaDTO> robotNoLimitationArea = getRobotNoLimitationArea(robots);
        Map<String, TaskRobotNoLimittationAreaDTO> robotNoLimittationAreaDTOMap =
                robotNoLimitationArea.stream().collect(Collectors.toMap(v -> v.getRobotNo(), Function.identity()));

        List<TaskToPathPlanningDTO> pathPlanningList = new ArrayList<>();
        List<RobotTaskDetailAddVO> taskDetails = new ArrayList<>();
        List<RobotTaskDO> tasks = new ArrayList<>();
        for (PositionMapItemDO v : robotMapItems) {
            RobotTaskDO task = new RobotTaskDO();
            String incrementByKey = redisUtil.getIncrementByKey(RobotCacheLockEnum.MOVE_TASK_NO.getKey());
            task.setTaskNo(moveTaskNo + DateUtils.getYearMonthDay() + incrementByKey);
            task.setId(IdUtil.getSnowflakeNextId());
            task.setCycleNumber(0L);
            task.setRemainingCycleNumber(0L);
            task.setStartTime(LocalDateTime.now());
            tasks.add(task);

            RobotTaskDetailAddVO detailAddVO = new RobotTaskDetailAddVO();
            detailAddVO.setId(IdUtil.getSnowflakeNextId());
            detailAddVO.setRobotTaskId(task.getId());
            detailAddVO.setTaskType(RobotTaskTypeEnum.MOVE_TO_POINT.getType());
            detailAddVO.setReleaseType(ReleaseTakeEnum.TO_LOCATION.getType());
            detailAddVO.setReleaseId(v.getId());
            detailAddVO.setToLocationNo(sortMap.get(v.getId()) + "");
            detailAddVO.setRobotNo(v.getRobotNo());
            taskDetails.add(detailAddVO);

            TaskToPathPlanningDTO pathPlanning = TaskToPathPlanningDTO.builder()
                    .orderId(detailAddVO.getId() + "")
                    .orderType(PathTaskTypeEnum.MOVE_TO_WAIT.getType())
                    .priority(1l)
                    .createTime(LocalDateTime.now())
                    .taskType(PathTaskTypeToRobotEnum.MOVE.getType())
                    .waitIds(Collections.singletonList(v.getId() + ""))
                    .build();

            TaskRobotNoLimittationAreaDTO taskRobotNoLimittationAreaDTO = robotNoLimittationAreaDTOMap.get(v.getRobotNo());
            List<TaskRobotNoLimittationAreaDTO> robotNoLimitions = Arrays.asList(taskRobotNoLimittationAreaDTO);
            pathPlanning.setRobotNoLimitationAreaDTOS(robotNoLimitions);

            String key = PathPlanningChcheConstant.PATH_PLANNING_TASK + pathPlanning.getOrderId();
            redisUtil.set(key, JSON.toJSONString(pathPlanning), taskChcheTime);
            pathPlanningList.add(pathPlanning);
        }

        robotTaskMapper.insertBatch(tasks);

        List<RobotTaskDetailDO> bean = BeanUtils.toBean(taskDetails, RobotTaskDetailDO.class);
        robotTaskDetailMapper.insertBatch(bean);

        positionMapItemMapper.updateById(robotMapItems);

        log.info("派车去停车点--任务下发给PP :{}", JSON.toJSONString(pathPlanningList));
        commonApi.commonMethod(pathPlanningList, PathPlanningTopicConstant.TASK_ASSIGNMENT_REQUEST);

    }


    @Override
    public void sendChargeTaskToPP(List<RobotChargeLogDO> logs, List<RobotTaskDetailDO> taskDetailDOS,
                                   List<RobotInformationDO> robots) {

        //机器人不能行走的区域
        List<TaskRobotNoLimittationAreaDTO> robotNoLimitationArea = getRobotNoLimitationArea(robots);

        Map<String, TaskRobotNoLimittationAreaDTO> robotNoLimittationAreaDTOMap =
                robotNoLimitationArea.stream().collect(Collectors.toMap(v -> v.getRobotNo(), Function.identity()));

        Map<String, RobotInformationDO> robotMap =
                robots.stream().collect(Collectors.toMap(v -> v.getRobotNo(), Function.identity()));

        Map<Long, RobotTaskDetailDO> taskDetailMap =
                taskDetailDOS.stream().collect(Collectors.toMap(v -> v.getId(), Function.identity()));

        List<TaskToPathPlanningDTO> pathPlanningList = new ArrayList<>();

        for (RobotChargeLogDO v : logs) {

            Long priority = priorityConfig;
            LocalDateTime createTime = LocalDateTime.now();
            if (ObjectUtil.isEmpty(ChargeModelEnum.TASK.getType().equals(v.getChargeModel()))
                    && ObjectUtil.isNotEmpty(v.getTaskDetailId())
                    && ObjectUtil.isNotEmpty(taskDetailMap.get(v.getTaskDetailId()))) {
                createTime = taskDetailMap.get(v.getTaskDetailId()).getCreateTime();
                priority = taskDetailMap.get(v.getTaskDetailId()).getPriority();
            }

            String orderType = ObjectUtil.isEmpty(v.getTaskDetailId()) ? PathTaskTypeEnum.AUTO_CHARGE.getType() : PathTaskTypeEnum.CHARGE.getType();
            TaskToPathPlanningDTO pathPlanning = TaskToPathPlanningDTO.builder()
                    .orderId(String.valueOf(v.getId()))
                    .orderType(orderType)
                    .priority(priority)
                    .createTime(createTime)
                    .build();

            TaskRobotNoLimittationAreaDTO taskRobotNoLimittationAreaDTO = robotNoLimittationAreaDTOMap.get(v.getRobotNo());
            List<TaskRobotNoLimittationAreaDTO> robotNoLimitions = Arrays.asList(taskRobotNoLimittationAreaDTO);
            pathPlanning.setRobotNoLimitationAreaDTOS(robotNoLimitions);

            RobotInformationDO robotInformationDO = robotMap.get(taskRobotNoLimittationAreaDTO.getRobotNo());
            String taskType = ChargeTypeEnum.AUTOMATIC.getType().equals(robotInformationDO.getChargeType()) ?
                    PathTaskTypeToRobotEnum.AUTO_CHARGE.getType() : PathTaskTypeToRobotEnum.MANUAL_CHARGE.getType();
            pathPlanning.setTaskType(taskType);

            pathPlanning.setReleaseGroupId("POINT_" + v.getPositionMapItemId());
            pathPlanning.setReleaseLocationNumber(releaseLocationNumberConfig);
            pathPlanning.setWaitIds(Collections.singletonList(v.getPositionMapItemId() + ""));

            String key = PathPlanningChcheConstant.PATH_PLANNING_TASK + pathPlanning.getOrderId();
            redisUtil.set(key, JSON.toJSONString(pathPlanning), taskChcheTime);

            pathPlanningList.add(pathPlanning);
        }

        log.info("充电任务下发给PP :{}", JSON.toJSONString(pathPlanningList));
        commonApi.commonMethod(pathPlanningList, PathPlanningTopicConstant.TASK_ASSIGNMENT_REQUEST);
    }

    @Transactional(rollbackFor = Exception.class)
    public void distributeTasksToPP(List<RobotInformationDO> robots, List<RobotTaskDetailDO> taskDetailDOS) {
        //目前一个点位/线库， 只能派一辆车去做
        taskDetailDOS = getSingleTask(taskDetailDOS);

        if (ObjectUtil.isEmpty(taskDetailDOS)) {
            log.info("相同的点位/线库，目前有处理中的任务, 无法派单给PP");
            return;
        }

        List<TaskToPathPlanningDTO> pathPlanningList = new ArrayList<>();

        Set<Long> locationIds = new HashSet<>();
        Set<Long> fromLocationIds = taskDetailDOS.stream().map(RobotTaskDetailDO::getFromLocationId).collect(Collectors.toSet());
        Set<Long> toLocationIds = taskDetailDOS.stream().map(RobotTaskDetailDO::getToLocationId).collect(Collectors.toSet());
        locationIds.addAll(fromLocationIds);
        locationIds.addAll(toLocationIds);

        List<WareHouseLocationDO> locations = locationMapper.selectList(new LambdaQueryWrapperX<WareHouseLocationDO>()
                .in(WareHouseLocationDO::getId, locationIds));

        Map<Long, WareHouseLocationDO> locationDOMap =
                locations.stream().collect(Collectors.toMap(v -> v.getId(), Function.identity()));

        //能执行此任务的机器人 和 不能走的区域
        Pair<List<TaskRobotNoLimittationAreaDTO>, Map<String, TaskRobotNoLimittationAreaDTO>> pair = getRobotLimitationArea(robots);
        Map<String, TaskRobotNoLimittationAreaDTO> robotDoReleaseMap = pair.getRight();
        List<TaskRobotNoLimittationAreaDTO> robotNoLimitationArea = pair.getLeft();

        //前一个任务是仅取货
        List<String> robotDoTake = getRobotDoTake(robots);

        List<PositionMapItemDO> positionMapItems = positionMapItemMapper.selectList(new LambdaQueryWrapperX<PositionMapItemDO>()
                .eq(PositionMapItemDO::getType, PositionMapItemEnum.STOP.getType())
                .eq(PositionMapItemDO::getUseStatus, UseStatusEnum.FREE.getType()));
        List<String> waitIds = null;
        if (ObjectUtil.isNotEmpty(positionMapItems)) {
            waitIds = positionMapItems
                    .stream()
                    .map(u -> u.getId() + "")
                    .collect(Collectors.toList());
        }
        Integer i = 0;

        log.info("开始组装需要下发给PP的任务");
        for (RobotTaskDetailDO taskDetailDO : taskDetailDOS) {

            String taskType = PathTaskTypeToRobotEnum.taskDetailGetType(taskDetailDO.getTaskType());
            TaskToPathPlanningDTO pathPlanning = TaskToPathPlanningDTO.builder()
                    .orderId(String.valueOf(taskDetailDO.getId()))
                    .orderType(PathTaskTypeEnum.getTaskType(taskDetailDO.getTaskType()))
                    .priority(taskDetailDO.getPriority())
                    .taskType(taskType)
                    .createTime(taskDetailDO.getCreateTime())
                    .build();

            if (RobotTaskTypeEnum.PARK.getType().equals(taskDetailDO.getTaskType())
                    && (ObjectUtil.isEmpty(waitIds) || waitIds.size() < i)) {
                log.info("停车任务,没有多余的空闲点位 ：{}", taskDetailDO.getId());
                return;
            } else if (RobotTaskTypeEnum.PARK.getType().equals(taskDetailDO.getTaskType())) {
                pathPlanning.setWaitIds(waitIds);
                i++;
            } else if (RobotTaskTypeEnum.MOVE_TO_POINT.getType().equals(taskDetailDO.getTaskType())) {
                moveToPoint(pathPlanning, taskDetailDO);
            }

            if (ObjectUtil.isNotEmpty(robotDoTake) && ObjectUtil.isNotEmpty(taskDetailDO.getRobotNo())
                    && robotDoTake.contains(taskDetailDO.getRobotNo())
                    && !RobotTaskTypeEnum.RELEASE.getType().equals(taskDetailDO.getTaskType())) {
                log.info("机器人前一个任务是仅取货，当前任务非仅放货,所以当前任务不执行 :{}", taskDetailDO.getId());
                continue;
            }

            List<TaskRobotNoLimittationAreaDTO> robotNoLimitions = null;
            if (ObjectUtil.isEmpty(taskDetailDO.getRobotNo())) {
                robotNoLimitions = robotNoLimitationArea;
            } else {
                TaskRobotNoLimittationAreaDTO taskRobotNoLimittationAreaDTO = robotDoReleaseMap.get(taskDetailDO.getRobotNo());
                robotNoLimitions = Arrays.asList(taskRobotNoLimittationAreaDTO);
            }

            if (ObjectUtil.isEmpty(robotNoLimitions)) {
                log.info("此任务明细,已经指定了机器人编号, 没有对应的机器人能执行此任务 :{}, 机器人编号:{}", taskDetailDO.getId(), taskDetailDO.getRobotNo());
                continue;
            }
            pathPlanning.setRobotNoLimitationAreaDTOS(robotNoLimitions);

            WareHouseLocationDO fromLocation = locationDOMap.get(taskDetailDO.getFromLocationId());
            WareHouseLocationDO toLocation = locationDOMap.get(taskDetailDO.getToLocationId());

            if (ObjectUtil.isNotEmpty(taskDetailDO.getFromLocationId())) {
                pathPlanning.setTakeLocationNumber(Math.abs(locationNumberReduce - taskDetailDO.getFromLocationNumber()));
                pathPlanning.setTakePointId(fromLocation.getMapItemId());

                pathPlanningSetTakeHeight(pathPlanning, fromLocation);
                pathPlanning.setTakeLevel(fromLocation.getLocationStorey());
                pathPlanning.setTakeOffsetHeight(offsetHeight);
                if (ObjectUtil.isNotEmpty(taskDetailDO.getFromLaneId())) {
                    pathPlanning.setTakeGroupId("LINE_" + taskDetailDO.getFromLaneId());
                } else {
                    pathPlanning.setTakeGroupId("POINT_" + taskDetailDO.getFromMapItemId());
                }
            }

            if (ObjectUtil.isNotEmpty(taskDetailDO.getToLocationId())) {
                pathPlanning.setReleaseLocationNumber(taskDetailDO.getToLocationNumber());
                pathPlanning.setReleasePointId(toLocation.getMapItemId());

                pathPlanningSetReleaseHeight(pathPlanning,toLocation);
                pathPlanning.setReleaseLevel(toLocation.getLocationStorey());
                pathPlanning.setReleaseOffsetHeight(offsetHeight);
                if (ObjectUtil.isNotEmpty(taskDetailDO.getToLaneId())) {
                    pathPlanning.setReleaseGroupId("LINE_" + taskDetailDO.getToLaneId());
                } else {
                    pathPlanning.setReleaseGroupId("POINT_" + taskDetailDO.getToMapItemId());
                }
            }

            String key = PathPlanningChcheConstant.PATH_PLANNING_TASK + pathPlanning.getOrderId();
            redisUtil.set(key, JSON.toJSONString(pathPlanning), taskChcheTime);

            pathPlanningList.add(pathPlanning);
        }

        if (ObjectUtil.isNotEmpty(pathPlanningList)) {
            log.info("作业任务下发给PP :{}", JSON.toJSONString(pathPlanningList));
            commonApi.commonMethod(pathPlanningList, PathPlanningTopicConstant.TASK_ASSIGNMENT_REQUEST);
        }
    }

    /**
     * 设置放货高度
     * @param pathPlanning
     * @param toLocation
     */
    private void pathPlanningSetReleaseHeight(TaskToPathPlanningDTO pathPlanning, WareHouseLocationDO toLocation) {
        if ( ZeroOneEnum.ONE.getType().equals(toLocation.getLocationStorey())) {
            pathPlanning.setReleaseHeight(0.0);
            return;
        }

        WareHouseLocationDO nextLocation = locationMapper.selectOne(new LambdaQueryWrapperX<WareHouseLocationDO>()
                .eq(WareHouseLocationDO::getId, toLocation.getMapItemId())
                .eq(WareHouseLocationDO::getLocationStorey, toLocation.getLocationStorey()-1)
                .last("limit 1"));
        if (ObjectUtil.isNotEmpty(nextLocation) && ObjectUtil.isNotEmpty(nextLocation.getLocationTotalHeight())) {
            pathPlanning.setReleaseHeight(Double.valueOf(nextLocation.getLocationTotalHeight() + ""));
            return;
        }

        Integer locationStorey = toLocation.getLocationStorey() - 1;
        double height = locationStorey * defaultTrayHeight;
        log.info("放货设置默认高度 :{}",pathPlanning.getReleaseHeight());
        pathPlanning.setReleaseHeight(height);
    }

    /**
     * 设置取货高度
     *
     * @param pathPlanning
     * @param fromLocation
     */
    private void pathPlanningSetTakeHeight(TaskToPathPlanningDTO pathPlanning, WareHouseLocationDO fromLocation) {
        if (ObjectUtil.isNotEmpty(fromLocation.getLocationTotalHeight())) {
            pathPlanning.setTakeHeight(Double.valueOf(fromLocation.getLocationTotalHeight() + ""));
            return;
        }
        Integer locationStorey = fromLocation.getLocationStorey();
        double height = locationStorey * defaultTrayHeight;
        log.info("取货设置默认取货高度 :{}",height);
        pathPlanning.setTakeHeight(height);
    }

    /**
     * 移动到路径点
     *
     * @param pathPlanning
     * @param taskDetailDO
     */
    private void moveToPoint(TaskToPathPlanningDTO pathPlanning, RobotTaskDetailDO taskDetailDO) {
        pathPlanning.setReleaseLocationNumber(1l);
        pathPlanning.setReleasePointId(taskDetailDO.getReleaseId());
        pathPlanning.setReleaseGroupId("POINT_" + taskDetailDO.getReleaseId());
    }

    /**
     * 获取能执行此任务的机器人 和 不能走的区域
     *
     * @param robots
     * @return
     */
    private Pair<List<TaskRobotNoLimittationAreaDTO>, Map<String, TaskRobotNoLimittationAreaDTO>> getRobotLimitationArea(List<RobotInformationDO> robots) {
        List<PositionMapDO> positionMapDOS = positionMapMapper.selectList(new LambdaQueryWrapperX<PositionMapDO>());

        List<TaskRobotNoLimittationAreaDTO> robotNoLimitationAreaDTOS = new ArrayList<>();
        Map<String, TaskRobotNoLimittationAreaDTO> map = new HashMap<>();
        for (RobotInformationDO robot : robots) {
            TaskRobotNoLimittationAreaDTO robotNoLimitationAreaDTO = new TaskRobotNoLimittationAreaDTO();
            List<TaskLimitationAreaDTO> limitationAreaList = new ArrayList<>();
            robotNoLimitationAreaDTO.setRobotNo(robot.getRobotNo());
            for (PositionMapDO v : positionMapDOS) {
                if (!robot.getFloorAreaJson().contains(v.getId())) {
                    TaskLimitationAreaDTO taskLimitationAreaDTO = new TaskLimitationAreaDTO(v.getFloor(), v.getArea());
                    limitationAreaList.add(taskLimitationAreaDTO);
                }
            }

            robotNoLimitationAreaDTO.setLimitationAreaList(limitationAreaList);

            if (RobotStatusEnum.LAST_TASK_IS_TAKE.getType().equals(robot.getRobotStatus())) {
                map.put(robot.getRobotNo(), robotNoLimitationAreaDTO);
            } else {
                robotNoLimitationAreaDTOS.add(robotNoLimitationAreaDTO);
                map.put(robot.getRobotNo(), robotNoLimitationAreaDTO);
            }

        }
        return ImmutablePair.of(robotNoLimitationAreaDTOS, map);
    }

    /**
     * 前一个任务是仅取货
     *
     * @param robots
     */
    private List<String> getRobotDoTake(List<RobotInformationDO> robots) {
        return robots.stream()
                .map(v -> {
                    if (v.getRobotStatus().equals(RobotStatusEnum.LAST_TASK_IS_TAKE.getType())) {
                        return v.getRobotNo();
                    }
                    return null;
                })
                .collect(Collectors.toList());
    }

    /**
     * 设置机器人编号及不能行走的区域
     *
     * @param robots
     */
    @Override
    public List<TaskRobotNoLimittationAreaDTO> getRobotNoLimitationArea(List<RobotInformationDO> robots) {
        List<PositionMapDO> positionMapDOS = positionMapMapper.selectList(new LambdaQueryWrapperX<PositionMapDO>());

        List<TaskRobotNoLimittationAreaDTO> robotNoLimitationAreaDTOS = new ArrayList<>();
        for (RobotInformationDO robot : robots) {

            TaskRobotNoLimittationAreaDTO robotNoLimitationAreaDTO = new TaskRobotNoLimittationAreaDTO();
            List<TaskLimitationAreaDTO> limitationAreaList = new ArrayList<>();
            robotNoLimitationAreaDTO.setRobotNo(robot.getRobotNo());

            for (PositionMapDO v : positionMapDOS) {
                if (!robot.getFloorAreaJson().contains(v.getId())) {
                    TaskLimitationAreaDTO taskLimitationAreaDTO = new TaskLimitationAreaDTO(v.getFloor(), v.getArea());
                    limitationAreaList.add(taskLimitationAreaDTO);
                }
            }

            robotNoLimitationAreaDTO.setLimitationAreaList(limitationAreaList);
            robotNoLimitationAreaDTOS.add(robotNoLimitationAreaDTO);
        }
        return robotNoLimitationAreaDTOS;
    }

    /**
     * 目前一个点位/线库， 只能派一辆车去做
     *
     * @param taskDetailDOS
     * @return
     */
    private List<RobotTaskDetailDO> getSingleTask(List<RobotTaskDetailDO> taskDetailDOS) {
        //剔除目前正在做的线库和点位
        if (!taskNeedSingle) {
            return taskDetailDOS;
        }

        log.info("---------处理:一个点位/线库 只派一台车 开始-------------");

        //不能分配任务的线库id
        Set<Long> laneIds = new HashSet<>();
        //不能分配任务的点位id
        Set<Long> mapIds = new HashSet<>();
        List<RobotTaskDetailDO> doingTaskDetails = robotTaskDetailMapper.selectList(new LambdaQueryWrapperX<RobotTaskDetailDO>()
                .eq(RobotTaskDetailDO::getTaskStatus, RobotTaskDetailStatusEnum.DOING.getType()));
        if (ObjectUtil.isNotEmpty(doingTaskDetails)) {
            for (RobotTaskDetailDO detail : doingTaskDetails) {
                if (RobotTaskTypeEnum.TAKE_RELEASE.getType().equals(detail.getTaskType())
                        || RobotTaskTypeEnum.MOVE.getType().equals(detail.getTaskType())
                        || RobotTaskTypeEnum.TAKE.getType().equals(detail.getTaskType())
                        || RobotTaskTypeEnum.RELEASE.getType().equals(detail.getTaskType())) {
                    if (ObjectUtil.isNotEmpty(detail.getFromLaneId())) {
                        laneIds.add(detail.getFromLaneId());
                    }
                    if (ObjectUtil.isNotEmpty(detail.getToLaneId())) {
                        laneIds.add(detail.getToLaneId());
                    }
                    if (ObjectUtil.isNotEmpty(detail.getFromMapItemId())) {
                        mapIds.add(detail.getFromMapItemId());
                    }
                    if (ObjectUtil.isNotEmpty(detail.getToMapItemId())) {
                        mapIds.add(detail.getToMapItemId());
                    }
                }
            }
        }

        //过滤处理中的, 不能分配的任务;
        List<RobotTaskDetailDO> detailDOS = taskDetailDOS.stream()
                .filter(v -> !laneIds.contains(v.getFromLaneId())
                        && !laneIds.contains(v.getToLaneId())
                        && !mapIds.contains(v.getFromMapItemId())
                        && !mapIds.contains(v.getToMapItemId()))
                .collect(Collectors.toList());

        if (ObjectUtil.isEmpty(detailDOS)) {
            log.info("过滤处理中的任务对应的线库id和点位id后，任务数据为空");
            return detailDOS;
        }

        //取货线库
        Map<Long, RobotTaskDetailDO> fromLaneMap = detailDOS.stream()
                .filter(v -> ObjectUtil.isNotEmpty(v.getFromLaneId()))
                .collect(Collectors.groupingBy(RobotTaskDetailDO::getFromLaneId,
                        Collectors.collectingAndThen(Collectors.minBy(Comparator.comparingLong(RobotTaskDetailDO::getFromLocationNumber)), Optional::get)));

        //放货线库
        Map<Long, RobotTaskDetailDO> toLaneMap = detailDOS.stream()
                .filter(v -> ObjectUtil.isNotEmpty(v.getToLaneId()))
                .collect(Collectors.groupingBy(RobotTaskDetailDO::getToLaneId,
                        Collectors.collectingAndThen(Collectors.maxBy(Comparator.comparingLong(RobotTaskDetailDO::getToLocationNumber)), Optional::get)));

        //取货点位--最小
        Map<Long, Long> fromLocationMap = detailDOS.stream()
                .filter(v -> ObjectUtil.isNotEmpty(v.getFromMapItemId()))
                .collect(Collectors.groupingBy(RobotTaskDetailDO::getFromMapItemId,
                        Collectors.collectingAndThen(Collectors.minBy(Comparator.comparingLong(RobotTaskDetailDO::getFromLocationNumber)),
                                a -> a.isPresent() ? a.get().getFromLocationNumber() : null)));

        //放货点位--最大
        Map<Long, Long> toLocationMap = detailDOS.stream()
                .filter(v -> ObjectUtil.isNotEmpty(v.getToMapItemId()))
                .collect(Collectors.groupingBy(RobotTaskDetailDO::getToMapItemId,
                        Collectors.collectingAndThen(Collectors.maxBy(Comparator.comparingLong(RobotTaskDetailDO::getToLocationNumber)),
                                a -> a.isPresent() ? a.get().getToLocationNumber() : null)));

        //返回的数据
        List<RobotTaskDetailDO> list = new ArrayList<>();
        for (RobotTaskDetailDO v : detailDOS) {
            RobotTaskDetailDO fromLane = ObjectUtil.isNotEmpty(v.getFromLaneId()) ? fromLaneMap.get(v.getFromLaneId()) : null;
            RobotTaskDetailDO toLane = ObjectUtil.isNotEmpty(v.getToLaneId()) ? toLaneMap.get(v.getToLaneId()) : null;
            Long fromLocation = ObjectUtil.isNotEmpty(v.getFromMapItemId()) ? fromLocationMap.get(v.getFromMapItemId()) : null;
            Long toLocation = ObjectUtil.isNotEmpty(v.getToMapItemId()) ? toLocationMap.get(v.getToMapItemId()) : null;
            //校验放货线库，有取的任务
            RobotTaskDetailDO fromLaneDoRelease = ObjectUtil.isNotEmpty(v.getToLaneId()) ? fromLaneMap.get(v.getToLaneId()) : null;
            //校验放的普通库位，有取的任务
            Long toHaveFrom = ObjectUtil.isNotEmpty(v.getToMapItemId()) ? fromLocationMap.get(v.getToMapItemId()) : null;
            //校验取货库位，有放的任务
            Long fromHaveTo = ObjectUtil.isNotEmpty(v.getFromMapItemId()) ? toLocationMap.get(v.getFromMapItemId()) : null;

            //取放模式
            if (RobotTaskTypeEnum.TAKE_RELEASE.getType().equals(v.getTaskType())) {
                takeReleaseSetTask(list, v, fromLane, toLane, fromLocation, toLocation, fromLaneDoRelease, toHaveFrom, fromHaveTo);
            } else if (RobotTaskTypeEnum.MOVE.getType().equals(v.getTaskType())
                    || RobotTaskTypeEnum.RELEASE.getType().equals(v.getTaskType())) {
                //移动和仅放货
                moveSetTask(list, v, fromLane, toLane, fromLocation, toLocation, toHaveFrom, fromHaveTo, fromLaneDoRelease);
            } else if (RobotTaskTypeEnum.TAKE.getType().equals(v.getTaskType())) {
                //仅取货
                takeSetTask(list, v, fromLane, toLane, fromLocation, toLocation, toHaveFrom, fromHaveTo);
            } else if (RobotTaskTypeEnum.PARK.getType().equals(v.getTaskType())
                    || RobotTaskTypeEnum.MOVE_TO_POINT.getType().equals(v.getTaskType())) {
                list.add(v);
            }
        }

        log.info("---------处理:一个点位/线库 只派一台车 结束------------- :{}", JSON.toJSONString(list));

        return list;
    }


    /**
     * 仅取货设置任务
     *
     * @param list
     * @param v
     * @param fromLane
     * @param toLane
     * @param fromLocation
     * @param toLocation
     */
    private void takeSetTask(List<RobotTaskDetailDO> list, RobotTaskDetailDO v, RobotTaskDetailDO fromLane,
                             RobotTaskDetailDO toLane, Long fromLocation, Long toLocation,
                             Long toHaveFrom, Long fromHaveTo) {

        if (ObjectUtil.isNotEmpty(fromHaveTo) && fromHaveTo < v.getFromLocationNumber()) {
            log.info("下发任务给PP--仅取货任务--取货点有放货任务 :{}", v.getId());
            return;
        }

        if (!(ObjectUtil.isNotEmpty(fromLocation) && fromLocation.equals(v.getFromLocationNumber()))) {
            log.info("下发任务给PP--仅取货--排序不符合 :{}", v.getId());
            return;
        }

        //终点是线库
        if (ObjectUtil.isNotEmpty(v.getFromLaneId()) && ObjectUtil.isNotEmpty(fromLane)
                && fromLane.getFromLocationNumber().equals(v.getFromLocationNumber())) {
            list.add(v);

            //终点是普通库位
        } else if (ObjectUtil.isEmpty(v.getFromLaneId())) {
            list.add(v);
        }

    }

    /**
     * 移动设置任务
     *
     * @param list
     * @param v
     */
    private void moveSetTask(List<RobotTaskDetailDO> list, RobotTaskDetailDO v, RobotTaskDetailDO fromLane,
                             RobotTaskDetailDO toLane,
                             Long fromLocation, Long toLocation, Long toHaveFrom,
                             Long fromHaveTo, RobotTaskDetailDO fromLaneDoRelease) {

        if (ObjectUtil.isNotEmpty(toHaveFrom) && toHaveFrom < v.getToLocationNumber()) {
            log.info("下发任务给PP--移动/仅放任务--放货点有取货任务 :{}", v.getId());
            return;
        }

        if (!(ObjectUtil.isNotEmpty(toLocation) && toLocation.equals(v.getToLocationNumber()))) {
            log.info("下发任务给PP--移动任务--排序不符合 :{}", v.getId());
            return;
        }

        //终点是线库
        if (ObjectUtil.isNotEmpty(v.getToLaneId()) && ObjectUtil.isNotEmpty(toLane)
                && toLane.getToLocationNumber().equals(v.getToLocationNumber())
                && ObjectUtil.isEmpty(fromLaneDoRelease)) {
            list.add(v);

            //终点是普通库位
        } else if (ObjectUtil.isEmpty(v.getToLaneId())) {
            list.add(v);
        }
    }


    /**
     * 取放，设置任务
     */
    private void takeReleaseSetTask(List<RobotTaskDetailDO> list, RobotTaskDetailDO v, RobotTaskDetailDO fromLane,
                                    RobotTaskDetailDO toLane, Long fromLocation, Long toLocation,
                                    RobotTaskDetailDO fromLaneDoRelease, Long toHaveFrom,
                                    Long fromHaveTo) {

        if (ObjectUtil.isNotEmpty(toHaveFrom) && toHaveFrom < v.getToLocationNumber()) {
            log.info("下发任务给PP--取放任务--放货点有取货任务 :{}", v.getId());
            return;

        } else if (ObjectUtil.isNotEmpty(fromHaveTo) && fromHaveTo < v.getFromLocationNumber()) {
            log.info("下发任务给PP--取放任务--取货点有放货任务 :{}", v.getId());
            return;
        }

        if (!(ObjectUtil.isNotEmpty(fromLocation) && fromLocation.equals(v.getFromLocationNumber())
                && ObjectUtil.isNotEmpty(toLocation) && toLocation.equals(v.getToLocationNumber()))) {
            log.info("下发任务给PP--取放任务--取/放的排序不符合 :{}", v.getId());
            return;
        }
        //取放都是线库
        if (ObjectUtil.isNotEmpty(v.getFromLaneId()) && ObjectUtil.isNotEmpty(v.getToLaneId())
                && ObjectUtil.isNotEmpty(fromLane)
                && fromLane.getFromLocationNumber().equals(v.getFromLocationNumber())
                && ObjectUtil.isNotEmpty(toLane)
                && toLane.getToLocationNumber().equals(v.getToLocationNumber())) {

            if (v.getFromLaneId().equals(v.getToLaneId())) {
                list.add(v);
            } else if (ObjectUtil.isEmpty(fromLaneDoRelease)) {
                //放货的线库，有取的任务，先不进行放
                list.add(v);
            }

            //取是线库，放是点位
        } else if (ObjectUtil.isNotEmpty(v.getFromLaneId()) && ObjectUtil.isEmpty(v.getToLaneId())
                && ObjectUtil.isNotEmpty(fromLane)
                && fromLane.getFromLocationNumber().equals(v.getFromLocationNumber())) {
            list.add(v);

            //取是点位，放是线库
        } else if (ObjectUtil.isEmpty(v.getFromLaneId()) && ObjectUtil.isNotEmpty(v.getToLaneId())
                && ObjectUtil.isNotEmpty(toLane)
                && toLane.getToLocationNumber().equals(v.getToLocationNumber())
                && ObjectUtil.isEmpty(fromLaneDoRelease)) {
            list.add(v);

            //取放都是点位
        } else if (ObjectUtil.isEmpty(v.getFromLaneId()) && ObjectUtil.isEmpty(v.getToLaneId())) {
            list.add(v);
        }
    }


}
